##LibName#Wilcoxon-Verteilung
##LibDescription#Die Bibliothek enthlt Funktionen zur Berechnung 
##LibDescription#der Dichte, Verteilungsfunktion und Quantile der
##LibDescription#Wilcoxon-Verteilung mit der richtigen Rangsumme.
##LibDescription#Modifizierter Wilcoxon-Rangsummen-Test
##LibCopyright#Ronja Foraita




dwil <- function (x,n,m) {
##FunctionName#dwil
##FunctionDescription#Diese Funktion erzeugt basierend auf der konkreten
##FunctionDescription#Rangsumme (=Pruefgroesse) die richtigen
##FunctionDescription#Werte der Dichte.
##FunctionSyntax#dwil(x , n , m )

	# kleinste Rangsumme berechnen

	  n.new <- n*(n+1)/2

	# von der konkreten Rangsumme (z.B. Pruefgroesse) n.new abziehen

	  w <- x-n.new

	# Dichte der Verteilung berechnen

	  dwilcox(w,n,m)
}




pwil <- function (q,n,m) {
##FunctionName#pwil
##FunctionDescription#Diese Funktion erzeugt basierend auf der konkreten
##FunctionDescription#Rangsumme (=Pruefgroesse) die richtigen
##FunctionDescription#Werte der Wahrscheinlichkeitsverteilung.
##FunctionSyntax#pwil(q , n , m )

	# kleinste Rangsumme berechnen

	  n.new <- n*(n+1)/2

	# von der konkreten Rangsumme (z.B. Pruefgroesse) n.new abziehen

	  w <- q-n.new

	# Wahrscheinlichkeit der Verteilungsfunktion berechnen
	
	  pwilcox(w,n,m)
}



qwil <- function (p,n,m){
##FunctionName#qwil
##FunctionDescription#Diese Funktion erzeugt basierend auf der konkreten
##FunctionDescription#Rangsumme (=Pruefgroesse) die richtigen
##FunctionDescription#Quantile der Verteilung.
##FunctionSyntax#qwil(p , n , m )

	# Quantil nach R-Logik berechnen
 	  
	  qw <- qwilcox(p,n,m)

	# zu dem Quantil die kleinst mgliche Rangsumme addieren
	
	  qw.new <- qw + n*(n+1)/2

}




wil.test <- function(x, y, alternative = "two.sided", a = 0.05){
##FunctionName#wil.test
##FunctionDescription#Diese Funktion berechnet den Wilcoxon-Rangsummen-Test
##FunctionDescription#auf Basis der konkreten Rangsummen.
##FunctionSyntax#wil.test(x , y , alternative=c("t","g","l"),a=0.05 )

	n <- length(x)
	m <- length(y)
	xn <- c(rep(1, n))
	ym <- c(rep(2, m))
	daten <- c(x, y)
	gruppe <- c(xn, ym)
	rang <- rank(daten)	



	# Pruefgrsse

		pg <- sum(rang[gruppe == 1])
		pg.g <- sum(rang[gruppe == 2])	



	# kritische Werte je nach Hypothese.
	
		if(alternative == "less" || alternative == "l") {
			q <- qwil(a, n, m)
		}
		if(alternative == "greater" || alternative == "g") {
			q <- qwil(1 - a, n, m)
		}
		if(alternative == "two.sided" || alternative == "t") {
			q1 <- qwil(a/2, n, m)
			q2 <- qwil(1 - a/2, n, m)
		}



	# p-Werte
		if(alternative == "less" || alternative == "l") {
			p <- pwil(pg, n, m)
		}
		if(alternative == "greater" || alternative == "g") {
			p <- pwil(pg.g, m, n)
		}
		if(alternative == "two.sided" || alternative == "t") {
			if(pg < (q1 + q2)/2) {
				p <- 2 * pwil(pg, n, m)
			}
		else {
				p <- 2 * pwil(pg.g, m, n)
			}
		}



	# Ausgabe
	cat("\n\n", "          Exakter Wilcoxon-Rangsummen-Test", "\n\n")
	cat("Stichprobenumfang: n =", n, ", m =", m, "\n")
	cat("Signifikanzniveau:", a, "\n")
	cat("  Pruefgroesse W :", pg, "\n")
	if(alternative == "greater" || alternative == "g") {
		cat("Alternativhypoth.: true mu is greater than 0", "\n")
		cat("  kritischer Wert:", q, "\n")
	}
	if(alternative == "less" || alternative == "l") {
		cat("Alternativhypoth.: true mu is less than 0", "\n")
		cat("  kritischer Wert:", q, "\n")
	}
	if(alternative == "two.sided" || alternative == "t") {
		cat("Alternativhypoth.: true mu is not equal to 0", "\n")
		cat("  kritische Werte:", q1, "<->", q2, "\n")
	}
	cat("           P-Wert:", round(p, digits = 5), "\n")
	
}
